"""
Some examples of using keyword arguments and writing functions that support
keyword arguments with kw** (supplementary material to Matplotlib unit)
"""

import csv

# end is a supported keyword in built-in print method
print('hello', end='')

# We could potentially implement this as:
# def print(s, kw**):
#   if 'end' in kw:
#     # print with no \n at end
#   else:
#     # print with default \n at end

with open('pts.csv', 'w') as csvfile:
  pt_data = [{'x' : 1, 'y' : 2}, {'x' : 2, 'y' : 3}]
  # Example of kw args in csv.Dictwriter
  # fieldnames=[...] populates the DictWriter's supported **kw argument
  writer = csv.DictWriter(csvfile, fieldnames=['x', 'y'])
  writer.writeheader()
  for row in pt_data:
    writer.writerow(row)


# Example implementing our own keyword args support
# need ** before kw to indicate to Python that it is a function
# supporting key=value arguments after the required x, y args
def foo(x, y, **kw):
  print(x, y, kw)

foo(1, 2, fieldnames=['x', 'y'], width='100', height='200')
# 1 2 {'fieldnames': ['x', 'y'], 'width': '100', 'height': '200'}
foo('bar', 'baz')
# 'bar' 'baz' {}
