"""
Lecture Demos Introducing Matplotlib Basics and Patterns.
You are encouraged to experiment around with these!

Students are strongly recommended to review the posted lecture materials
to learn the fundamental patterns/conventions
of Matplotlib and the key components of any Matplotlib.pyplot program. 
A recording will also be provided, reviewing strategies for experimenting,
learning, and debugging with plt in VSCode.
"""
import matplotlib.pyplot as plt


def demo1():
    # "3 Ways to Plot" Slide

    # plt.subplots() returns a (Figure, Axes) tuple
    # Tuple unpacking convention for plt.subplots
    (fig, ax) = plt.subplots()

    # If nrow, ncols arguments are given, axs will be a 
    # list of nrow * ncols Axes attached to the single Figure
    # (fig, axs) = plt.subplots(1, 2)
    # (fig, axs) = plt.subplots(2, 2)
    plt.show()


def demo2():
    (fig, axs) = plt.subplots(1, 2)
    # Note: If fig is not used, you can use _ wildcard just like
    # an unused variable in a loop
    # (_, axs) = plt.subplots(1, 2)

    # Tuple unpacking for two Axes plots
    (ax1, ax2) = axs

    # Basic Line Graphs (2 lines plotted on first Axes graph)
    # Exercise: What do you think the arguments represent?
    ax1.plot([1, 2, 3], [1, 2, 3], color='red')
    ax1.plot([1, 2, 3], [3, 2, 1], 
      	 	 color='#00ff00') # green

    # 1 line plotted on second Axes graph
    ax2.plot([1, 2, 3], [0, 3, 0], 'o-', color='green', linewidth=5)
    plt.show()


def demo3():
    # Experimenting with line graphs and plotted points
    # Recommended to watch Lecture 17 video on getting used
    # to the syntax with Matplotlib plots!
    xs = [1, 2, 3]
    ys1 = [0, 3, 1]
    ys2 = [1, 4, 2]

    (fig, ax1) = plt.subplots()
    
    # plot x and y using blue lines
    ax1.plot(xs, ys1, 'b--')  
    # ditto, but with red stars
    ax1.plot(xs, ys2, 'r*-')  
    
    (x, y) = (2, 3)
    # plot (x, y) using blue circle markers
    ax1.plot(x, y, 'bo')  
    # ditto, but with red stars  
    ax1.plot(x, y, 'r*')  
    
    ax1.plot(ys1, xs, color='#00ff00', linewidth=5)
    ax1.plot(2, 3, color='#d14900') # Caltech orange!
    
    plt.show()


def spotify_bar_chart_demo():
    # A simple bar chart demo using the first 8 songs in the
    # CS 1 staff playlist
    songs = ['N95', '505', "World's Smallest Violin", 'Thnks fr th Mmrs', 
             'Headlines', 'West Coast', 'Shook Ones, Pt. II', 
             "Don't Wanna Fall In Love"]
    popularities = [83, 82, 78, 78, 77, 76, 76, 73]
    (fig, ax) = plt.subplots()
    fig.set_size_inches(8, 7)

    ax.bar(songs, popularities, color='blue', width=0.6)

    # Abbreviate songs if we want to override tick labels
    # (easier with fewer bars)
    for i in range(len(songs)):
        abbr = songs[i]
        if len(abbr) > 5:
            abbr = abbr[:5] + '...'
            songs[i] = abbr

    # Override the x tick labels to use the song name strings
    # and rotate by 45 degrees
    ax.set_xticklabels(songs, rotation=45)
    plt.show()


def spotify_pie_chart_demo():
    """
    Demo using a pie chart, simple version of Lab04's pie chart
    """
    genre_frequencies = {'rock': 3, 'pop': 1, 
                         'hip hop': 2, 'classical': 4}
    labels = genre_frequencies.keys()
    frequencies = genre_frequencies.values()

    fig, ax = plt.subplots()
    fig.set_size_inches(4, 4)
    # In Lab04, legend was plotted as separate Axes to 
    # have more control of space (but could alternatively use .legend)
    ax.pie(frequencies, labels=labels)
    ax.set_title('Playlist Genre Frequencies')
    plt.show()


if __name__ == '__main__':
    demo1()
    # demo2()
    # demo3()
    # spotify_bar_chart_demo()
    # spotify_pie_chart_demo()