"""
Some examples of CSV processing from 22fa Lectures.
"""

import csv

# Functions added to generate counts for matplotlib demo
def count_years():
    """
    Returns a (str, int) tuple of (years, counts) for # of students
    in each year ('Freshman', 'Sophomore', 'Junior', Senior', 'Other')
    """
    categories = ['Freshman', 'Sophomore', 'Junior', 'Senior']
    counts = [0, 0, 0, 0]
    total = 0
    STUDENT_DATA = 'cs1_22fa_students.csv'
    with open(STUDENT_DATA) as f:
        reader = csv.DictReader(f)
        for row in reader:
            total += 1
            for i, category in enumerate(categories):
                if category in row['Year']:
                    counts[i] += 1
                    break
    other = total - sum(counts)
    counts.append(other)
    categories.append('Other')
    return (categories, counts)


def count_lab_pref():
    """
    Returns a (str, int) tuple of (pref, counts) for # of students
    with indicated preferences of Individual, Group, Both,
    or No Preference.
    """
    categories = ['Individual', 'Group', 'Both', 'No Preference']
    counts = [0, 0, 0, 0]
    total = 0
    STUDENT_DATA = 'cs1_22fa_students.csv'
    with open(STUDENT_DATA) as f:
        reader = csv.DictReader(f)
        for row in reader:
            total += 1
            pref = row['Lab Preferences'].lower()
            if 'both' in pref or ('individual' in pref and 'group' in pref):
                counts[2] += 1
            elif 'individual' in pref:
                counts[0] += 1
            elif 'group' in pref:
                counts[1] += 1
            else:
                counts[3] += 1
    return (categories, counts)


def percentage_intended(major_keywords):
    """
    Answers the question 'what percentage of the class is intended
    to have one of the passed intended major keywords.
    Example: percentage_intended(['CS', 'CNS', 'IDS', 'Computation'])'

    Returns:
        - (float) - percentage between 0.0 and 100.0
    """
    STUDENT_DATA = 'cs1_22fa_students.csv'
    total = 0
    count = 0
    # 1. Get the data
    with open(STUDENT_DATA, 'r') as csvfile:
        reader = csv.DictReader(csvfile)
        # 2. Analyze the dataset
        for row in reader:
            intended_major = row['Intended major']
            # Update the running count if the student has a computational
            # major noted
            # substr in full_str -> True if substr is _somewhere_ in full_str
            for keyword in major_keywords:
                if keyword in intended_major:
                    count += 1
                    break
            # Update the total count
            total += 1
        return count / total * 100


def percentage_cs_intended():
    """
    Answers the question 'what percentage of the class is intended
    CS (CS, CNS, IDS, Computation)?'

    Returns:
        - (float) - percentage between 0.0 and 100.0
    """
    SEARCH_KEYWORDS = ['CS', 'CNS', 'IDS', 'Computation']
    result = percentage_intended(SEARCH_KEYWORDS)
    return result


def percentage_undecided():
    """
    Answers the question 'what percentage of the class is undecided?'

    In this interpretation, we consider any student who has the word
    'Undecided' or ' or ' in their responses as undecided.
    """
    SEARCH_KEYWORDS = ['Undecided', ' or ']
    result = percentage_intended(SEARCH_KEYWORDS)
    return result


def count_years():
    categories = ['Freshman', 'Sophomore', 'Junior', 'Senior']
    counts = [0, 0, 0, 0]
    total = 0
    STUDENT_DATA = 'cs1_22fa_students.csv'
    with open(STUDENT_DATA) as f:
        reader = csv.DictReader(f)
        for row in reader:
            total += 1
            for i, category in enumerate(categories):
                if category in row['Year']:
                    counts[i] += 1
                    break
    other = total - sum(counts)
    counts.append(other)
    categories.append('Other')
    return (categories, counts)


def count_lab_pref():
    categories = ['Individual', 'Group', 'Both', 'No Preference']
    counts = [0, 0, 0, 0]
    total = 0
    STUDENT_DATA = 'cs1_22fa_students.csv'
    with open(STUDENT_DATA) as f:
        reader = csv.DictReader(f)
        for row in reader:
            total += 1
            pref = row['Lab Preferences'].lower()
            if 'both' in pref or ('individual' in pref and 'group' in pref):
                counts[2] += 1
            elif 'individual' in pref:
                counts[0] += 1
            elif 'group' in pref:
                counts[1] += 1
            else:
                counts[3] += 1
    return (categories, counts)


if __name__ == '__main__':
    percent_cs = percentage_cs_intended()
    percent_undecided = percentage_undecided()

    print(f'{percent_cs:.2f}% of CS 1 22fa students are intended CS-related.')
    print(f'{percent_undecided:.2f}% of CS 1 22fa students are undecided.')


# Extra questions to practice:
# nobel_prizes.csv
# 1. Write a function to print the name and motivation for all winners of
# a specific category
# 2. Write a function to return a list of all names of winners of a specific
# year

# dictionary.csv
# 1. Write a function to return the definitions of a given word,
# ignoring letter-casing
# Can you think of others?

def nobel_winners_for(category):
    # print(f'{row["firstname"]} {row["surname"]} ({row["year"]})')
    pass

# nobel_winners_for('physics')
